@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2012	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -762,6 +762,7 @@ static void daemonize(void)
 	dup2(i, 0);
 	dup2(i, 1);
 	dup2(i, 2);
+	close(i);
 
     fdlimit = sysconf(_SC_OPEN_MAX);
     for (i = 3; i < fdlimit; i++)
@@ -1329,7 +1330,6 @@ static RETSIGTYPE exit_handler(int sig)
 	{
 		pool_debug("exit_handler: I am not parent");
 		POOL_SETMASK(&UnBlockSig);
-		pool_shmem_exit(0);
 		exit(0);
 	}
 
@@ -1348,6 +1348,11 @@ static RETSIGTYPE exit_handler(int sig)
 
 	exiting = 1;
 
+	/* Close listen socket */
+	pool_log("pgpool main: close listen socket");
+	close(inet_fd);
+	close(unix_fd);
+
 	for (i = 0; i < pool_config->num_init_children; i++)
 	{
 		pid_t pid = process_info[i].pid;
@@ -2348,45 +2353,53 @@ static int find_primary_node(void)
 
 		status = do_query(con, "SELECT count(*) FROM pg_catalog.pg_proc AS p WHERE p.proname = 'pgpool_walrecrunning'",
 						  &res, PROTO_MAJOR_V3);
-		if (res->numrows <= 0)
+		
+		if(status == POOL_CONTINUE)
 		{
-			pool_log("find_primary_node: do_query returns no rows");
-		}
-		if (res->data[0] == NULL)
-		{
-			pool_log("find_primary_node: do_query returns no data");
-		}
-		if (res->nullflags[0] == -1)
-		{
-			pool_log("find_primary_node: do_query returns NULL");
-		}
-		if (res->data[0] && !strcmp(res->data[0], "0"))
-		{
-			pool_log("find_primary_node: pgpool_walrecrunning does not exist");
-			free_select_result(res);
-			discard_persistent_db_connection(s);
-			return -1;
+			if (res->numrows <= 0)
+			{
+				pool_log("find_primary_node: do_query returns no rows");
+			}
+			if (res->data[0] == NULL)
+			{
+				pool_log("find_primary_node: do_query returns no data");
+			}
+			if (res->nullflags[0] == -1)
+			{
+				pool_log("find_primary_node: do_query returns NULL");
+			}
+			if (res->data[0] && !strcmp(res->data[0], "0"))
+			{
+				pool_log("find_primary_node: pgpool_walrecrunning does not exist");
+				free_select_result(res);
+				discard_persistent_db_connection(s);
+				return -1;
+			}
 		}
 
 		status = do_query(con, "SELECT pg_is_in_recovery() AND pgpool_walrecrunning()",
 						  &res, PROTO_MAJOR_V3);
-		if (res->numrows <= 0)
+		if(status == POOL_CONTINUE)
 		{
-			pool_log("find_primary_node: do_query returns no rows");
+			if (res->numrows <= 0)
+			{
+				pool_log("find_primary_node: do_query returns no rows");
+			}
+			if (res->data[0] == NULL)
+			{
+				pool_log("find_primary_node: do_query returns no data");
+			}
+			if (res->nullflags[0] == -1)
+			{
+				pool_log("find_primary_node: do_query returns NULL");
+			}
+			if (res->data[0] && !strcmp(res->data[0], "t"))
+			{
+				is_standby = true;
+			}
 		}
-		if (res->data[0] == NULL)
-		{
-			pool_log("find_primary_node: do_query returns no data");
-		}
-		if (res->nullflags[0] == -1)
-		{
-			pool_log("find_primary_node: do_query returns NULL");
-		}
-		if (res->data[0] && !strcmp(res->data[0], "t"))
-		{
-			is_standby = true;
-		}   
-		free_select_result(res);
+		if(res)
+			free_select_result(res);
 		discard_persistent_db_connection(s);
 
 		/*
