@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2011	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -96,6 +96,7 @@ void pool_start_query(POOL_QUERY_CONTEXT *query_context, char *query, Node *node
 		query_context->rewritten_query = NULL;
 		query_context->parse_tree = node;
 		query_context->virtual_master_node_id = REAL_MASTER_NODE_ID;
+		query_context->num_original_params = -1;
 		pool_set_query_in_progress();
 		session_context->query_context = query_context;
 	}
@@ -165,6 +166,14 @@ void pool_clear_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 {
 	int i;
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+	{
+		pool_error("pool_setall_node_to_be_sent: no session context");
+		return;
+	}
 
 	if (!query_context)
 	{
@@ -176,7 +185,20 @@ void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 	{
 		if ((BACKEND_INFO(i)).backend_status == CON_UP ||
 			(BACKEND_INFO((i)).backend_status == CON_CONNECT_WAIT))
+		{
+			/*
+			 * In streaming replication mode, if the node is not
+			 * primary node nor load balance node, there's no point to
+			 * send query.
+			 */
+			if (pool_config->master_slave_mode &&
+				!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
+				i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
+			{
+				continue;
+			}
 			query_context->where_to_send[i] = true;
+		}
 	}
 	return;
 }
